package NET.worlds.scape;

import NET.worlds.network.URL;
import java.util.Enumeration;
import java.util.Vector;

public class ShapeLoader
  implements BGLoaded
{
  Shape shape;
  int binaryParam;
  private int numTexturesLoading;
  private Vector textures;
  private boolean wasError;
  
  public ShapeLoader(Shape s)
  {
    this.shape = s;
  }
  
  public Room getBackgroundLoadRoom()
  {
    return this.shape.getRoom();
  }
  
  public Object asyncBackgroundLoad(String localName, URL remoteName)
  {
    if (localName == null) {
      return null;
    }
    if ((remoteName.endsWith(".rwg")) || (remoteName.endsWith(".RWG")))
    {
      this.binaryParam = loadBinaryFile(localName, remoteName);
      return new Object();
    }
    if ((remoteName.endsWith(".rwx")) || (remoteName.endsWith(".RWX")))
    {
      loadTextFile(localName, remoteName);
      return localName;
    }
    if ((remoteName.endsWith(".bod")) || (remoteName.endsWith(".BOD"))) {
      return localName;
    }
    return null;
  }
  
  public boolean syncBackgroundLoad(Object o, URL remoteURL)
  {
    if (o == null) {
      return false;
    }
    if (this.numTexturesLoading > 0) {
      return true;
    }
    if (!remoteURL.equals(this.shape.realFile))
    {
      this.wasError = true;
    }
    else if ((!this.shape.hasClump()) && (this.shape.isDefault))
    {
      this.wasError = true;
      if (!this.shape.isLoaded()) {
        this.shape.setState(0, null);
      }
    }
    int newModel;
    if ((o instanceof String))
    {
      if (!this.wasError)
      {
        if (remoteURL.endsWith(".bod")) {
          newModel = loadBodFile((String)o, this.shape.getBodPartNum());
        } else {
          newModel = finishLoadingTextFile((String)o);
        }
      }
      else
      {
        newModel = 0;
      }
    }
    else
    {
      newModel = finishLoadingBinaryFile(this.binaryParam, this.wasError);
    }
    if (newModel != 0) {
      this.shape.setState(newModel, this.textures);
    }
    if ((newModel == 0) || (newModel == -1)) {
      NET.worlds.console.Console.println("Shape " + remoteURL.toString() + " failed to load. Code: " + newModel);
      if (this.textures != null) {
        for (Enumeration en = this.textures.elements(); en.hasMoreElements();) {
          ((Texture)en.nextElement()).decRef();
        }
      }
    }
    this.textures = null;
    
    return false;
  }
  
  private void startTextureLoad(String relName, URL remoteName)
  {
    this.numTexturesLoading += 1;
    BackgroundLoader.get(new ShapeTextureLoader(this), 
      URL.make(remoteName, relName), 
      true);
  }
  
  synchronized void textureLoadEnd(Texture tex)
  {
    this.numTexturesLoading -= 1;
    if (tex != null)
    {
      if (this.textures == null) {
        this.textures = new Vector();
      }
      this.textures.addElement(tex);
    }
    else
    {
      this.wasError = true;
    }
  }
  
  private native void loadTextFile(String paramString, URL paramURL);
  
  private native int finishLoadingTextFile(String paramString);
  
  private native int loadBodFile(String paramString, int paramInt);
  
  private native int loadBinaryFile(String paramString, URL paramURL);
  
  private native int finishLoadingBinaryFile(int paramInt, boolean paramBoolean);
}
